<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

/*
 * Load Additional Data when getting Order Items by Repository 
 */
class ItemGet
{
    /** @var \FiloBlu\OrderAdditionalData\Model\ItemDataFactory */
    protected $itemDataFactory;

    /** @var \Magento\Sales\Api\Data\ItemExtensionFactory */
    protected $itemExtensionFactory;

    /**
     * Init plugin
     *
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $itemExtensionFactory
     */
    public function __construct(
        \FiloBlu\OrderAdditionalData\Model\ItemDataFactory $itemDataFactory,
        \Magento\Sales\Api\Data\OrderItemExtensionFactory $itemExtensionFactory
    ) {
        $this->itemDataFactory = $itemDataFactory;
        $this->itemExtensionFactory = $itemExtensionFactory;     
    }

    /**
     * Get additional data
     *
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderItemInterface $resultItem
     * @return \Magento\Sales\Api\Data\OrderItemInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderItemInterface $resultItem
    ) {
        $resultItem = $this->getItemAdditionalData($resultItem);
        return $resultItem;
    }

    /**
     * Get additional data for item
     *
     * @param \Magento\Sales\Api\Data\OrderItemInterface $order
     * @return \Magento\Sales\Api\Data\OrderItemInterface
     */
    protected function getItemAdditionalData(\Magento\Sales\Api\Data\OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();
        if ($extensionAttributes && $extensionAttributes->geFbAdditionalData()) {
            return $item;
        }
        //$additionalData=$this->objectFactory->create();
        $additionalData=array();
        try {
            /** @var \Magento\AdditionalData\Api\Data\MessageInterface $AdditionalData */
            $itemDataObject = $this->itemDataFactory->create()->load($item->getItemId(), "item_id");

            if ($itemDataObject && $itemDataObject->getId() > 0 && $itemDataObject->getAdditionalData()!=null ){
                $additionalData=unserialize($itemDataObject->getAdditionalData());
            }
        } catch (NoSuchEntityException $e) {
            return $item;
        }
        /** @var \Magento\Sales\Api\Data\ItemExtension $itemExtension */
        $itemExtension = $extensionAttributes ? $extensionAttributes : $this->itemExtensionFactory->create();
        $itemExtension->setFbAdditionalData(array($additionalData));
        $itemExtension->setData("kpi_full_price", floatval($item->getData("kpi_full_price")));
        $item->setExtensionAttributes($itemExtension);
        return $item;
    }

    /**
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param \Magento\Sales\Model\ResourceModel\Order\Item\Collection $resultOrder
     * @return \Magento\Sales\Model\ResourceModel\Order\Item\Collection
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        \Magento\Sales\Model\ResourceModel\Order\Item\Collection $collection
    ) {
        /** @var  $item */
        foreach ($collection->getItems() as $item) {
            $this->afterGet($subject, $item);
        }
        return $collection;
    }
}
